/*
 * $ROOTpark: tools/nwn/nwnusers/nwnusers.c,v 1.4 2003/02/02 01:17:18 aheil Exp $
 */

#include <libgen.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

int main(int argc, char **argv) {
	char *host;
	unsigned int port;
	unsigned int timeout;
	int sockfd;
	struct sockaddr_in their_addr;
	struct hostent *he;
	fd_set fds;
	struct timeval tv;
	char buffer[4096];
	char question[6];
	int c;

	host = strdup("localhost"); /* default host */
	port = 5121;                /* default port */
	timeout = 5;                /* default timeout is 5 seconds */

	while((c = getopt(argc, argv, "hp:t:v")) != -1) {
		switch(c) {
			case 'h':
				printf("%s %s\n", basename(argv[0]), VERSION);
				printf("Copyright (C) ROOTpark n.e.V.\n");
				printf("Written by Andreas Heil <ah@linux-hq.de>.\n");
				printf("Usage:\n");
				printf("    %s [-hv] [-p <port>] [-t <timeout>] [<host>]\n", basename(argv[0]));
				printf("Options:\n");
				printf("    -h              print help screen and exit\n");
				printf("    -p <port>       use udp port <port> for communication (default: %u)\n", port);
				printf("    -t <timeout>    set timeout to <timeout> seconds (default: %u)\n", timeout);
				printf("    -v              print version information and exit\n");
				printf("    <host>          connect to host <host> (default: %s)\n", host);
				printf("Report bugs to <nwnusers@linux-hq.de>.\n");
				exit(0);
			case 'p':
				port = atoi(optarg);
				break;
			case 't':
				timeout = atoi(optarg);
				break;
			case 'v':
				printf("%s %s\n", basename(argv[0]), VERSION);
				printf("Copyright (C) ROOTpark n.e.V.\n");
				printf("Written by Andreas Heil <ah@linux-hq.de>.\n");
				exit(0);
			case '?':
			default:
				fprintf(stderr, "Usage: %s [-hv] [-p <port>] [-t <timeout>] [<host>]\n", basename(argv[0]));
				exit(1);
		}
	}
	argc -= optind;
	argv += optind;

	if(argc >= 1 && strcmp(argv[0], "0")) {
		free(host);
		host = strdup(argv[0]);
	}
	
	if((he = gethostbyname(host)) == NULL) {
		printf("0\n0\n\n%s\n", hstrerror(h_errno));
		exit(1);
	}

	if((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
		printf("0\n0\n\n%s\n", strerror(errno));
		exit(1);
	}

	their_addr.sin_family = AF_INET;
	their_addr.sin_port = htons(port);
	their_addr.sin_addr = *((struct in_addr *)he->h_addr);
	memset(&(their_addr.sin_zero), '\0', 8);
	
	question[0] = 0x42;
	question[1] = 0x4e;
	question[2] = 0x58;
	question[3] = 0x49;
	question[4] = 0x00;
	question[5] = 0x14;

	if(sendto(sockfd, question, 6, 0, (struct sockaddr *)&their_addr, sizeof(struct sockaddr)) == -1) {
		printf("0\n0\n\n%s\n", strerror(errno));
		exit(1);
	}
	
	FD_ZERO(&fds);
	FD_SET(sockfd, &fds);
	tv.tv_sec = timeout;
	tv.tv_usec = 0;
	
	c = select(sockfd + 1, &fds, NULL, NULL, &tv);
	if(c == 0) {
		printf("0\n0\n\ntimeout: no response\n");
		exit(1);
	} else if(c == -1) {
		printf("0\n0\n\n%s\n", strerror(errno));
		exit(1);
	} else {
		if(FD_ISSET(sockfd, &fds)) {
			c = sizeof(struct sockaddr);
			recvfrom(sockfd, buffer, sizeof(buffer), 0, (struct sockaddr *)&their_addr, &c);
		}
	}

	close(sockfd);

	printf("%d\n%d\n\n%s\n", buffer[10], buffer[10], "Status OK");

	exit(0);
}
